/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGDEVICE_H_
#define FANCYDRAWINGDEVICE_H_

#include <fancydrawingglobals.h>
#include <fancyobject.h>

namespace Fancy {

class List;

namespace Drawing {

class Point2;
class XForm;
class Color;
class Image;
class TextBBox;

/**
 * An interface implemented by each physical device implementation.
 *
 * By default all coordinates, lengths ... are specified in postcript points, AKA "Pt".
 *
 * x is from left to right.
 * y is from bottom to top.
 * On an A4 page, these are the corner Point2's:
 *       0.0, 842.75     595.91, 842.75
 *       0.0,   0.0      595.91,   0.0
 *
 * To "switch" to millimeters (mm), call something like:
 * dev->scale(MM2PT(1.0));
 *
 * The, on an A4 page, these are the corner Point2's:
 *       0.0, 297.0      210.0, 297.0
 *       0.0,   0.0      210.0,   0.0
 *
 * MM2PT is a define in Fancy::Drawing::Helper, which provides some more helpful stuff btw.
 *
 */
class FANCYDRAWING_API Device : virtual public Fancy::Object {

public:
    typedef Fancy::ObjPtr<Device> ptr_t;

    /**
     * Start a page with agiven metric paper size.
     * WARNING: Only PDF files (i.e. PdfDevice) support different PaperSize and orientation per page.
     * PrinterDevice's pseudo implementiation for Unix only wraps PdfDevice + lpr, where it might work dependend on the printer driver's intelligence.
     * PrinterDevice's Windows implementation currently only supports general landscape orientation through PrinterSetup::setLandscape(bool)
     */
    virtual void beginPage(PaperSize p, bool landscape);
    /**
     * Start a page with a custom page size given in postscript points.
     */
    virtual void beginPage(const Point2 &sz) = 0;

    virtual void setLineWidth(double w) = 0;
    virtual void setCapStyle(CapStyle capStyle) = 0;
    virtual void setJoinStyle(JoinStyle joinStyle) = 0;
    virtual void setMiterLimit(double ml) = 0;
    virtual void setDashLengths(double offset, const double *dashes, int ndashes) = 0;
    virtual void setDashStyle(DashStyle ds, double dashLength);

    virtual void setStrokingColor(const Color &color) = 0;
    virtual void setFillColor(const Color &color) = 0;

    virtual void moveTo(const Point2 &p) = 0;
    virtual void lineTo(const Point2 &p) = 0;
    virtual void curveTo(const Point2 &p1, const Point2 &p2, const Point2 &p3) = 0;
    virtual void closePath() = 0;
    virtual void stroke() = 0;

    virtual void fill(bool winding) = 0;
    virtual void fillAndStroke(bool winding) = 0;

    virtual void clip(bool winding) = 0;

    /**
     * Render a single measured text lines as returned by TextBBox::newInstance()
     *
     * @param p The reference point to draw the text to.
     * @param text A pre-measured text bounding box.
     * @param halign The horizontal alignment w.r.t to p
     * @param halign The vertical alignment w.r.t to p
     */
    virtual void renderText(
            const Point2 &p,
            const Fancy::ObjPtr<TextBBox> &text,
            HorizonalAlignment halign,
            VerticalAlignment valign) = 0;

    /**
     * Render a set of measured text lines as returned by TextBBox::fitSize()
     *
     * @param p The reference point to draw the text to.
     * @param lineSkip The factor to multiply the nominal
     *          text size of line i with in order to calculate the
     *          advance in negative y direction from line i to line i+1.
     * @param textBBoxes A list of text bounding boxes.
     * @param halign The horizontal alignment w.r.t to p
     * @param halign The vertical alignment w.r.t to p
     */
    virtual void renderTexts(
            const Point2 &p,
            double lineSkip,
            const Fancy::ObjPtr<Fancy::List> &textBBoxes,
            HorizonalAlignment halign,
            VerticalAlignment valign) = 0;

    virtual void drawImage(
            const Point2& ll,
            const Point2& lr,
            const Point2 &ur,
            const Fancy::ObjPtr<Image> &image) = 0;

    virtual void save() = 0;
    virtual void restore() = 0;

    virtual void transform(const XForm &xform) = 0;

    // simple scaling. if facY is set 0.0, facX is used again
    virtual void scale(double facX, double facY = 0.0);

    virtual void translate(double dx, double dy);
    virtual void rotate(double angle);

protected:
    Device();
    virtual ~Device();
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGDEVICE_H_ */
