/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGHELPER_H_
#define FANCYDRAWINGHELPER_H_

#ifndef MM2PT
#define MM2PT(m) ((m)*2.83464566929133858267)
#endif
#ifndef PT2MM
#define PT2MM(p) ((p)*0.35277777777777777777)
#endif

#include <fancydrawing.h>

#include <fancydrawingglobals.h>
#include <fancydrawingpoint2.h>

#include <fancyobject.h>

#include <string>

namespace Fancy {

class ConfigOptions;

namespace Drawing {

class Device;
class Image;

namespace Helper {

// RECTANGLES

/*
 * Draws a rectangle with the specified width.
 *
 * Feel free to prepare dash stiles with calls like
 *   dev->setLineWidth()
 *   dev->setDashStyle()
 *   dev->setFillColor()
 */
void FANCYDRAWING_API drawRect(
        const Fancy::ObjPtr<Device> & device,
        double left,
        double top,
        double right,
        double bottom);

void FANCYDRAWING_API drawLine(
        const Fancy::ObjPtr<Device> & device,
        const Point2 & p1,
        const Point2 & p2);

// IMAGES

Fancy::ObjPtr<Image> FANCYDRAWING_API loadImage(
        const std::string & fileName);

void FANCYDRAWING_API drawImage(
        const Fancy::ObjPtr<Device> & device,
        const Fancy::ObjPtr<Image> & image,
        double left,
        double top,
        double right,
        double bottom);


// TEXT

// 9 alignemnt combos
enum HVAlignment {
    HVAlignTopLeft      = 1,
    HVAlignTopCenter    = 2,
    HVAlignTopRight     = 3,
    HVAlignMiddleLeft   = 4,
    HVAlignMiddleCenter = 5,
    HVAlignMiddleRight  = 6,
    HVAlignBottomLeft   = 7,
    HVAlignBottomCenter = 8,
    HVAlignBottomRight  = 9,
};

/**
 * If device is specifed non-null:
 *   Draws the text to the specified device.
 *   Returns: Alway true.
 *
 * If device is specified NULL:
 *   Uses TextBBox::fitSize() with one additional allowed line, as space test.
 *   Returns whether the space would be enough to print all text.
 */
bool FANCYDRAWING_API drawText(
        const Fancy::ObjPtr<Device> & device,
        const std::string & text,
        Font font,
        double fontSize,
        double left,
        double top,
        double right,
        double bottom,
        int maxLines = 0,
        double marginToFrame = 0.0,
        HVAlignment hvAlignment = HVAlignTopLeft,
        double lineSkip = 1.0,
        TextStyle textStyle = DefaultTextStyle);


// far more to come


// FILE PRINTING

/**
 * Prints a file.
 *
 * In Linux,
 *   lpr [-P printerName] file
 * is executed, which should be able to print pretty much anything.
 * No printer dialog will show up.
 *
 * In Windows, something like
 *   gswinXXc.exe -sDEVICE=mswinpr2 -dNoCancel -dNOPAUSE -dBATCH foobar.pdf
 * is called, which requires a recent enough ghostscript to be installed.
 * This should work for PDF and PS files, I have no idea about other file types.
 * Unless a printerName is specified to something gs can get a printer context from,
 * the Windows printer dialog will show up.
 *
 * printerName is optional
 *   If it's empty i.e. unspecified, lpr will use the default printer,
 *   while ghostscript will show a printer selection dialog.
 *
 * options is optional
 *   It may carry special hints.
 *   Known options are:
 *     "GSBIN"   full path of ghostscript commandline binary, i.e.
 *               C:\Progra~1\gs\gs9.16\bin\gswin32c.exe
 *
 *
 * returns: exit code of used tool, i.e. lpr or ghostscript. Hence, 0=OK.
 */
int FANCYDRAWING_API printFile(
        const std::string & filename,
        const std::string & printerName = "",
        const Fancy::ObjPtr<Fancy::ConfigOptions> & options = 0);


} /* namespace Helper */
} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGHELPER_H_ */
