/***********************************************************
 * $Id$
 *
 * FancyDrawing tests
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGIMAGE_H_
#define FANCYDRAWINGIMAGE_H_

#include <fancydrawing.h>
#include <fancyobject.h>

namespace Fancy {

class Map;
class String;

namespace Drawing {

class Color;

/**
 * An in-memory pixel image to be written to a file or
 * being read from a file.
 */
class FANCYDRAWING_API Image : public Fancy::Object {

public:

    /*
     * Meta info key for PNG compression. The value will
     * be a Fancy::Integer instance in the range from 1 to 9.
     */
    static const Fancy::ObjPtr<Fancy::String> COMPRESSION;
    /*
     * Meta info key for JPEG compression. The value will
     * be a Fancy::Integer instance in the range from 1 to 99.
     */
    static const Fancy::ObjPtr<Fancy::String> QUALITY;
    /*
     * Meta info key for an image comment. The value will
     * be a Fancy::String instance.
     */
    static const Fancy::ObjPtr<Fancy::String> COMMENT;

    /*
     * Meta info key for the physical resolution of this image
     * in X direction in pixels per inch (dpi). The value will
     * be a Fancy::Number instance.
     */
    static const Fancy::ObjPtr<Fancy::String> RESOLUTION_X;

    /*
     * Meta info key for the physical resolution of this image
     * in Y direction in pixels per inch (dpi). The value will
     * be a Fancy::Number instance.
     */
    static const Fancy::ObjPtr<Fancy::String> RESOLUTION_Y;

    /*
     * Meta info key for the original file format where the image
     * has been read from. The value is a string like "png" or "jpeg",
     * which may be used for ImageIO::newInstance().
     */
    static const Fancy::ObjPtr<Fancy::String> IO_FORMAT;

    /*
     * Meta info key for the original file name where the image
     * has been read from. This value might be used to read to
     * original, compressed image.
     */
    static const Fancy::ObjPtr<Fancy::String> IO_FILENAME;

    typedef Fancy::ObjPtr<Image> ptr_t;

    enum ImageFormat {
        ImageFormat8BitIndexed,
        ImageFormatGrey,
        ImageFormatGreyAlpha,
        ImageFormatRGB,
        ImageFormatRGBAlpha
    };

    /**
     * Construct a new, empty image. Either filled with transparent
     * or white, if no alpha channel is present.
     *
     * The pixel data is managed by ourselves.
     */
    static ptr_t newInstance(ImageFormat format, int width, int height);

    /**
     * Construct a new image pointing to an externally managed memory buffer.
     * @c stride specifies the number of bytes per row.
     *
     * The pixel data is never freed by our destructor.
     */
    static ptr_t newInstance(ImageFormat format, int width, int height, uint8_t *data, size_t stride);

    /**
     * If the passed image is an 8 bit indexed image, convert
     * it to an image in format @c ImageFormatGrey or
     * @c ImageFormatRGB.
     *
     * If the image is in another format, return the passed image
     * pointer without conversion.
     */
    static ptr_t expandIndexed(const ptr_t &image);

    /**
     * Copy RESOLUTION_X and RESOLUTION_Y from source to target, if
     * source contains these properties.
     */
    static void copyResolution(const Fancy::ObjPtr<Image> &target, const Fancy::ObjPtr<Image> &source);

    /**
     * @return the bit depth of the specified format.
     */
    static int getDepthOfFormat(ImageFormat format);

    /**
     * @return the format of this image as passed in @c newInstance()
     */
    ImageFormat getFormat() const {
        return this->format;
    }

    /**
     * @return the number of columns of this image.
     */
    int getWidth() const {
        return this->width;
    }

    /**
     * @return the number of rows of this image.
     */
    int getHeight() const {
        return this->height;
    }

    /**
     * @return the bit depth of the image.
     */
    int getDepth() const {
        return this->depth;
    }

    /**
     * @return The number of bytes per line.
     */
    size_t getStride() const {
        return this->stride;
    }

    /**
     * @return The total size of the data array, aka @c getStride()*getHeight()
     */
    size_t getDataSize() const {
        return this->stride * this->height;
    }

    /*
     * @return the data buffer of the image.
     */
    uint8_t *getData() const {
        return this->data;
    }

    /*
     * @return the data buffer of the given row of an image, aka
     *         @c getData()+getStride()*irow
     */
    uint8_t *getRowData(int irow) const {
        return this->data + this->stride*irow;
    }

    Fancy::ObjPtr<Fancy::Map> getMetaInfos() const;

    Fancy::ObjPtr<Fancy::Object> getMetaInfo(const Fancy::ObjPtr<Fancy::String> &key) const;
    Fancy::ObjPtr<Fancy::Object> getMetaInfo(const char *key) const;
    Fancy::ObjPtr<Fancy::Object> getMetaInfo(const std::string &key) const;

    void setMetaInfo(const char *key, const Fancy::ObjPtr<Fancy::Object> &info);
    void setMetaInfo(const std::string &key, const Fancy::ObjPtr<Fancy::Object> &info);
    void setMetaInfo(const Fancy::ObjPtr<Fancy::String> &key, const Fancy::ObjPtr<Fancy::Object> &info);

    void setMetaInfos(const Fancy::ObjPtr<Fancy::Map> &metaInfos);

    /**
     * Return true, if the image format is
     * ImageFormatGrey or ImageFormatGreyAlpha
     * or all palette colors are grey for images of
     * format ImageFormat8BitIndexed
     */
    bool isGrey() const;

    /**
     * Get the color of the pixel in the specified column and row.
     */
    virtual Color getPixelColor(int icol, int irow) const = 0;

    /**
     * Set the color of the pixel in the specified column and row.
     */
    virtual void setPixelColor(int icol, int irow, const Color &col) = 0;

    /**
     * Get the value of the pixel as defined by the image format.
     * The value will be a color index, a grey value, or
     * @c (alpha<<8)|grey or
     * @c (red<<16)|(green<<8)|blue
     * @c (alpha<<24)|(red<<16)|(green<<8)|blue
     */
    virtual uint32_t getPixelValue(int icol, int irow) const = 0;

    /**
     * Set the pixel value as defined by the image format.
     */
    virtual void setPixelValue(int icol, int irow, uint32_t v) = 0;

    /**
     * Return the number of color in the color palette or 0, if
     * the image format does not have a palette.
     */
    virtual int getPaletteSize() const = 0;

    /**
     * Return the palette color at the index @i
     * if the index is less than zero or greater than
     * the palette size, @ Color::TRANSPARENT is returned.
     */
    virtual Color getPaletteColor(int i) const = 0;

    /**
      * Set the palette color at the index @i
      * if the index is less than zero or greater than
      * the palette size, an exception is thrown.
      */
    virtual void setPaletteColor(int i, const Color &col) = 0;

protected:
    Image(ImageFormat _format, int depth, int width, int height, uint8_t *data, size_t stride, bool data_owned);
    virtual ~Image();

private:

    ImageFormat format;
    int depth;
    int width;
    int height;
    uint8_t *data;
    size_t stride;
    bool data_owned;
    Fancy::ObjPtr<Fancy::Map> metaInfos;
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGIMAGE_H_ */
