/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGPDFDEVICE_H_
#define FANCYDRAWINGPDFDEVICE_H_

#include <fancydrawingdevice.h>


namespace Fancy {

class ConfigOptions;

namespace Drawing {

/**
 * An interface implemented by each physical device implementation.
 *
 * This implementation embeds the font families Arial, CourierNew,
 * TimesNewRoman and Verdana as a subset of the TTF files found in
 * the well-known font paths
 *
 *   /usr/share/fonts/truetype/msttcorefonts
 *
 * under Linux or
 *
 *   %SystemRoot%\fonts\
 *
 * under Microsoft Windows.
 *
 * If you want to render texts using a fallback to the infamous
 * ArialUnicodeMS font, use the UnicodeFallback or UnicodeFallbackKerning
 * text style for constructing the TextBBox object to be rendered.
 *
 * Additionally, you have to install the ArialUnicode font file,
 * usually named Arial_Unicode.tff of ARIALUNI.TFF to the above mentioned
 * font directory. The Arial Unicode font is delivered alongside with a long
 * list of Microsoft product as listed under
 *
 *   http://www.microsoft.com/typography/fonts/font.aspx?FMID=1081
 *
 * The "traditional" font families Helvetica, Courier and Times rely on the
 * installed fonts of the PDF reader and they use encoding descriptions
 * based on PostSCript glyph names, which are deprecated and do not produce
 * reliable results for non-latin1 characters.
 *
 * So it is recommended to only use Helvetica, Courier and Times for rendering
 * latin1 texts. For rendering internationalized texts, use Arial et al.
 *
 * For very advanced unicode examples (Asian characters or
 * very exotic symbols), install ArialUnicodeMS and use the unicode fallback
 * text styles as described above.
 */
class FANCYDRAWING_API PdfDevice : virtual public Device {

public:
    typedef Fancy::ObjPtr<PdfDevice> ptr_t;

    /**
     * Construct a new in-memory PDF instance.
     * When finished with constructing the contents, call
     * @c getPdfResult() on the returned instance.
     */
    static ptr_t newInstance();

    /**
     * Construct a new instance, which writes to a file.
     * When finished with constructing the contents, call
     * @c close() on the returned instance.
     */
    static ptr_t newInstance(const std::string &filename);

    /**
     * Finalize the PDF structure and close the resulting file
     * for instances created using the filename-argument
     * @c newInstance() method.
     */
    virtual void close() = 0;

    /**
     * Finalize the PDF structure and return the in-memory
     * PDF result for instances created using
     * the no-argument @c newInstance() method.
     */
    virtual std::string getPdfResult()  = 0;


    // FIXME nextgen: evtl. move rest to Device

    // option names

    // int value to add '/Rotate %d' after /Page
    // ignored if empty or 0
#define FANCYDRAWING_PDFPAGEROTATION "PDF_Page_Rotation"

    /**
     * Allows to set special options for unusual situations.
     * See FANCY_OPTION_PDF* defines above for supported values
     */
    virtual void setOptions(const Fancy::ObjPtr<Fancy::ConfigOptions> &options) = 0;

    /**
     * Returns optons set by setOptions
     */
    virtual Fancy::ObjPtr<Fancy::ConfigOptions> getOptions() = 0;


protected:
    PdfDevice();
    virtual ~PdfDevice();
};

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPDFDEVICE_H_ */
