/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGPOINT2_H_
#define FANCYDRAWINGPOINT2_H_

#include <fancydrawing.h>
#include <iosfwd>
#include <math.h>

namespace Fancy {
namespace Drawing {

/**
 * A simple two-dimensional vector with all the arithmetics needed.
 */
class FANCYDRAWING_API Point2 {

    double x;
    double y;

public:
    Point2() : x(0.0),y(0.0) {}
    Point2(double _x, double _y) : x(_x),y(_y) {}
    Point2(const Point2 &p) : x(p.x),y(p.y) {}

    double getX() const  { return this->x; }
    double getY() const  { return this->y; }

    void setX(double _x) { this->x = _x; }
    void setY(double _y) { this->y = _y; }

    Point2 &operator= (const Point2 &p) { this->x=p.x; this->y=p.y; return *this; }
    Point2 &operator+= (const Point2 &p) { this->x+=p.x; this->y=+p.y; return *this; }
    Point2 &operator-= (const Point2 &p) { this->x-=p.x; this->y=-p.y; return *this; }
    Point2 &operator*= (double c) { this->x*=c; this->y*=c; return *this; }
    Point2 &multadd (double c, const Point2 &b) { this->x+=c*b.x; this->y+=c*b.y; return *this; }

    double getLength() const { return ::hypot(this->x,this->y); };
};

double distance(const Point2 &a, const Point2 &b) FANCYDRAWING_API;

std::ostream &operator<< (std::ostream &os, const Point2 &p) FANCYDRAWING_API;

inline double operator== (const Point2 &a, const Point2 &b) {
    return a.getX() == b.getX() && a.getY() == b.getY();
}

inline double operator!= (const Point2 &a, const Point2 &b) {
    return a.getX() != b.getX() || a.getY() != b.getY();
}

inline Point2 operator+ (const Point2 &a, const Point2 &b) {
    return Point2(a.getX()+b.getX(),a.getY()+b.getY());
}

inline Point2 operator- (const Point2 &a, const Point2 &b) {
    return Point2(a.getX()-b.getX(),a.getY()-b.getY());
}

inline double operator* (const Point2 &a, const Point2 &b) {
    return a.getX()*b.getX() + a.getY()*b.getY();
}

inline double cross (const Point2 &a, const Point2 &b) {
    return a.getX()*b.getY() - a.getY()*b.getX();
}

inline double angleBetween (const Point2 &a, const Point2 &b) {
    return ::atan2(cross(a,b),a*b);
}

inline Point2 operator* (const Point2 &a, double c) {
    return Point2(a.getX()*c,a.getY()*c);
}

inline Point2 operator* (double c, const Point2 &a) {
    return Point2(a.getX()*c,a.getY()*c);
}

inline Point2 operator/ (const Point2 &a, double c) {
    return a * (1.0/c);
}

// a + c*b
inline Point2 multadd (const Point2 &a, double c, const Point2 &b) {
    return Point2(a.getX()+c*b.getX(),a.getY()+c*b.getY());
}

// ca*a + cb*b
inline Point2 lincomb (double ca, const Point2 &a, double cb, const Point2 &b) {
    return Point2(ca*a.getX()+cb*b.getX(),ca*a.getY()+cb*b.getY());
}

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPOINT2_H_ */
