/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGPRINTERSETUP_H_
#define FANCYDRAWINGPRINTERSETUP_H_

#include <fancydrawing.h>
#include <fancyobject.h>

namespace Fancy {
namespace Drawing {

/**
 * Printer options to be passed to the printer dialog.
 *
 * @c pages is the number of pages of the document to be printed.
 *   If the number of page is unknown, set this value to the
 *   default value 0.
 *   If the selection of page ranges should be disabled, set
 *   this value to -1.
 *
 * @c copies is the initial number of copies to be selected.
 *   If the number is set to 0 or a negative value, the selection
 *   of copies by the user is hidden from the printer dialog.
 *
 * @c useSoftwareCollation If this value is set to the non-default
 *   value true, the application is responsible for collating
 *   multiple copies.
 *   The default value is false and signifies, that the printer
 *   driver is responsible for collating multiple copies, if the
 *   user selected to print multiple copies if @c copies was set to
 *   a value greater than 0.
 *
 * @c landscape If this value is set to the non-default value true,
 *   the printer setup dialog should propose landscape orientation.
 *   WARNING: This currently is only effective with PrinterDevice's Windows implementation.
 *   PdfDevice as well as PrinterDevice's pseudo implementiation for Unix only respect beginPage() calls.
 *
 *   Under Windows this is helpful for documents that create landscape oriented pages only.
 *   Documents with mixed orientation still rely on future work on sophisticated auto-rotating PrinterDevice implementations.
 */
class FANCYDRAWING_API PrinterSetup : virtual public Fancy::Object {

    int pages;
    int copies;
    bool useSoftwareCollation;
    bool landscape;

public:
    typedef Fancy::ObjPtr<PrinterSetup> ptr_t;

    static ptr_t newInstance();

    int getCopies() const {
        return this->copies;
    }

    void setCopies(int copies) {
        this->copies = copies;
    }

    int getPages() const {
        return this->pages;
    }

    void setPages(int pages) {
        this->pages = pages;
    }

    bool isUseSoftwareCollation() const {
        return this->useSoftwareCollation;
    }

    void setUseSoftwareCollation(bool useSoftwareCollation) {
        this->useSoftwareCollation = useSoftwareCollation;
    }

    bool isLandscape() const {
        return this->landscape;
    }

    void setLandscape(bool landscape) {
        this->landscape = landscape;
    }

    std::string toString() const;

protected:
    PrinterSetup();
    virtual ~PrinterSetup();
};


} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPRINTERSETUP_H_ */
