/***********************************************************
 * $Id$
 *
 * FancyDrawing
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDRAWINGXFORM_H_
#define FANCYDRAWINGXFORM_H_

#include <fancydrawingpoint2.h>

namespace Fancy {
namespace Drawing {

/**
 * A simple two-dimensional transformation matrix with all the arithmetics needed.
 *
 * This implementation obeys the rules defined in section
 * 4.2.3 Transformation Matrices of PDFReference16.pdf
 *
 * A coordinate transformation is performed by
 *
 * |x'|   | a  c |   | x |   | dx |   | a*x + c*y + dx |
 * |  | = |      | * |   | + |    | = |                |
 * |y'|   | b  d |   | y |   | dy |   | b*x + d*y + dy |
 *
 */
class FANCYDRAWING_API XForm {

    double a;
    double b;
    double c;
    double d;
    double dx;
    double dy;

public:
    XForm() : a(1.0),b(0.0),c(0.0),d(1.0),dx(0.0),dy(0.0) {}
    XForm(double _dx, double _dy) : a(1.0),b(0.0),c(0.0),d(1.0),dx(_dx),dy(_dy) {}
    XForm(double _a, double _b, double _c, double _d, double _dx, double _dy) : a(_a),b(_b),c(_c),d(_d),dx(_dx),dy(_dy) {}

    /*
     * initialize from a rotation angle in radians and a translation offset.
     */
    XForm(double angle, double _dx, double _dy);

    double getA() const  { return this->a; }
    double getB() const  { return this->b; }
    double getC() const  { return this->c; }
    double getD() const  { return this->d; }

    double getDx() const  { return this->dx; }
    double getDy() const  { return this->dy; }

    void setA(double _a) { this->a = _a; }
    void setB(double _b) { this->b = _b; }
    void setC(double _c) { this->c = _c; }
    void setD(double _d) { this->d = _d; }

    void setDx(double _dx) { this->dx = _dx; }
    void setDy(double _dy) { this->dy = _dy; }

    double getDeterminant() const { return this->a*this->d - this->b*this->c; };
};

std::ostream &operator<< (std::ostream &os, const XForm &xform) FANCYDRAWING_API;

/**
 * Transform point p by the given transformation matrix according to the
 * rules in PDFReference16.pdf
 */
inline Point2 operator* (const XForm &xform, const Point2 &p) {
    return Point2(
            xform.getDx() + xform.getA() * p.getX() + xform.getC() * p.getY(),
            xform.getDy() + xform.getB() * p.getX() + xform.getD() * p.getY()
    );
}

/**
 * Append the transformation x2 to the transformation x1
 */
inline XForm operator* (const XForm &x1, const XForm &x2) {
    return XForm(
            x1.getA() * x2.getA() + x1.getC() * x2.getB(), x1.getB() * x2.getA() + x1.getD() * x2.getB(),
            x1.getA() * x2.getC() + x1.getC() * x2.getD(), x1.getB() * x2.getC() + x1.getD() * x2.getD(),
            x1.getDx() + x1.getA() * x2.getDx() + x1.getC() * x2.getDy(),
            x1.getDy() + x1.getB() * x2.getDx() + x1.getD() * x2.getDy()
    );
}

} /* namespace Drawing */
} /* namespace Fancy */

#endif /* FANCYDRAWINGPOINT2_H_ */
